/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      GLite Data SRM API C module: SrmTest class
 *
 *      Authors: Paolo Badino <paolo.badino@cern.ch>
 *      Version info: $Id: SrmTest.c,v 1.8 2006/11/16 17:35:07 badino Exp $ 
 *      Release: $Name:  $
 *
 *      Revision history:
 *      $Log: SrmTest.c,v $
 *      Revision 1.8  2006/11/16 17:35:07  badino
 *      Require gsoap 2.7.6b
 *      Review Makefiles
 *
 *      Revision 1.7  2004/10/27 10:18:20  badino
 *      Add DeleteFile tool
 *
 *      Revision 1.6  2004/10/26 12:41:36  pkunszt
 *      add proper security handling to test
 *
 *      Revision 1.5  2004/09/28 10:45:35  badino
 *      Set SOAP Action to the default value in C test application examples
 *
 *      Revision 1.4  2004/09/23 14:01:22  badino
 *      Create a library for server APIs that could be useful for creating stubs for unit or functional tests
 *      Update to the new version of gsoap.m4 and glite-version.m4
 *
 *      Revision 1.3  2004/09/22 16:48:51  badino
 *      Update to the new version of gsoap.m4
 *
 *      Revision 1.2  2004/09/14 09:18:49  badino
 *      Use different GSOAP versions for wsdl2h and soapcpp2 compilers
 *
 *      Revision 1.1.1.1  2004/09/07 17:05:20  badino
 *      Import of GLite Data SRM API C module v 0.0
 *
 *
 *
 */
#include <stdlib.h>
#include <stdio.h>

#include "glite/data/srm/c/srmH.h"
#include "glite/data/srm/c/srm.nsmap"

/*
 * Include CGSI GSOAP plugin
 */
#include "cgsi_plugin.h"

/**
 * GSOAP Name Wrappers. This code is needed if you want to compile with 
 * GSOAP 2.3 and 2.6
 */
#if _GSOAP_WSDL2H_VERSION < 0x020600

#define srm__pingResponse    tns__pingResponse
#define soap_call_srm__ping  soap_call_tns__ping
/*
 * Wraps GSOAP 2.3 Meta names
 */
#else
/*
 * Wraps GSOAP 2.6 Meta names
 */
#endif

#if _GSOAP_VERSION < 0x020600
/*
 *  GSOAP 2.3 specifics
 */
#else
/*
 *  GSOAP 2.6 specifics
 */
#endif

static const char * PROGNAME   = "glite_data_srm_api_c_test";
static const char * PROGVERS   = "0.1";
static const char * AUTHOR     = "(C) Paolo Badino - paolo.badino@cern.ch";

/*
 * Forward Declaration
 */
int ping(const char * endpoint);
   
/*
 * print_usage
 * 
 * Print Usage
 */
void print_usage(){
    fprintf(stderr,"\n");
    fprintf(stderr,"<%s> Version %s by %s\n",PROGNAME,PROGVERS,AUTHOR);
    fprintf(stderr,"\nUsage:\n");
    fprintf(stderr,"%s -e endpoint\n",PROGNAME);
    fprintf(stderr,"where:\n");
    fprintf(stderr,"\tendpoint  : the SRM service endpoint\n");
    fprintf(stderr,"\n");
}
   
/*
 * main
 * 
 * Test main Function
 */
int main(int argc, char * argv[]){
 
    char *endpoint = 0;
    int c;
    while ((c = getopt (argc, argv, "he:")) != -1)
    switch (c){
        case 'e':
            endpoint = optarg;
        break;
        case 'h':
        default:
            print_usage();
            return -1;
        break;
    }
    if(0 == endpoint){
        print_usage();
        return -1;
    }

    fprintf(stdout,"contacting %s...\n", endpoint);
    int result = ping(endpoint);
    fprintf(stdout,"Ping %s : %s\n", endpoint, result == 0 ? "OK" : "FAILED");
    return result;
}

/*
 * ping
 * 
 * Invoke ping
 */
int ping(const char * endpoint){
      
    struct soap soap;
    soap_init (&soap);
    soap.namespaces = srm_namespaces;

    if(0 == strncasecmp(endpoint,"httpg://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK;
        soap_register_plugin_arg (&soap, client_cgsi_plugin, &flags);
    } else if(0 == strncasecmp(endpoint,"https://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_SSL_COMPATIBLE;
        soap_register_plugin_arg (&soap, client_cgsi_plugin, &flags);
    }
        
#if _GSOAP_WSDL2H_VERSION < 0x020700
    struct srm__pingResponse out;
#else
    enum xsd__boolean out;
#endif

    // Invoke Service ping
    // -------------------------------------------------------------------------
    if(SOAP_OK != soap_call_srm__ping(&soap, endpoint, NULL, &out)){
        fprintf(stderr,"Cannot Contact SRM Service: Error in srm__ping - %s\n",soap.fault->faultcode);
        soap_print_fault (&soap, stderr);
        return -1;
    }
    
    return 0;
}
