/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      GLite Data SRM API C module: SrmCreateFile class
 *
 *      Authors: Paolo Badino <paolo.badino@cern.ch>
 *      Version info: $Id: SrmCreateFile.cxx,v 1.8 2007/04/04 15:43:56 knienart Exp $ 
 *      Release: $Name:  $

 */
#include <stdlib.h>
#include <stdio.h>
#if _GSOAP_VERSION >= 0x020700
#include "glite/data/srm/cpp/envH.h"
#endif
#include "glite/data/srm/cpp/srmH.h"
#include "glite/data/srm/cpp/srm.nsmap"

/*
 * Include CGSI GSOAP plugin
 */
extern "C"{
#include "cgsi_plugin.h"
}

/**
 * GSOAP Name Wrappers. This code is needed if you want to compile with 
 * GSOAP 2.3 and 2.6
 */
#if _GSOAP_WSDL2H_VERSION < 0x020600

/*
 * Wraps GSOAP 2.3 File names
 */
#include "glite/data/srm/cpp/srmsoapSRMServerV1Proxy.h"
namespace srm{
typedef soapSRMServerV1                 ISRM;
typedef tns__putResponse                srm__putResponse;
typedef tns__getRequestStatusResponse   srm__getRequestStatusResponse;
typedef tns__setFileStatusResponse      srm__setFileStatusResponse;
typedef ns11__RequestStatus             srm2__RequestStatus;
typedef ns11__RequestFileStatus         srm2__RequestFileStatus;
}
#define GSOAP_RETVAL(a) &a
#define GSOAP_FIELDVAL(a,b) a.b

#else

#if _GSOAP_WSDL2H_VERSION >= 0x020700
namespace srm{
typedef srm4ArrayOfstring        ArrayOfstring;
typedef srm5ArrayOflong          ArrayOflong;
typedef srm5ArrayOfboolean       ArrayOfboolean;
}
#endif

/*
 * Wraps GSOAP 2.6 File names
 */
#include "glite/data/srm/cpp/srmISRMProxy.h"
#define GSOAP_RETVAL(a) a
#define GSOAP_FIELDVAL(a,b) a->b
#endif



#if _GSOAP_VERSION < 0x020600
/*
 *  GSOAP 2.3 specifics
 */
#define srm__put                        put
#define srm__getRequestStatus           getRequestStatus
#define srm__setFileStatus              setFileStatus

#else
/*
 *  GSOAP 2.6 specifics
 */
#endif

namespace {
const char * PROGNAME   = "glite_data_srm_api_cpp_test_create";
const char * PROGVERS   = "0.1";
const char * AUTHOR     = "(C) Paolo Badino - paolo.badino@cern.ch";
}

/*
 * Forward Declaration
 */
int create_file(const char * endpoint, const char * surl, LONG64 size, const char * protocol, bool translate_surl);
      
/*
 * print_usage
 * 
 * Print Usage
 */
void print_usage(){
    fprintf(stderr,"\n");
    fprintf(stderr,"<%s> Version %s by %s\n",PROGNAME,PROGVERS,AUTHOR);
    fprintf(stderr,"\nUsage:\n");
    fprintf(stderr,"%s -e endpoint -f <filename> -p <protocol> [-s <size>]\n",PROGNAME);
    fprintf(stderr,"where:\n");
    fprintf(stderr,"\t-e endpoint  : the SRM service endpoint\n");
    fprintf(stderr,"\t-f filename  : the name of file to create\n");
    fprintf(stderr,"\t-p protocol  : the protocol that will be used to access the file\n");
    fprintf(stderr,"\t-s size      : the file size\n");
    fprintf(stderr,"\t-n           : disable the SURL translation\n");
    fprintf(stderr,"\n");
}
   
/*
 * main
 * 
 * Test main Function
 */
int main(int argc, char * argv[]){
 
    char * endpoint  = 0;
    char * surl      = 0;
    char * protocol  = 0;
    LONG64 size      = 0;
    bool   translate_surl = true;
    
    int c;
    while ((c = getopt (argc, argv, "he:f:p:s:n")) != -1)
    switch (c){
        case 'e':
            endpoint = optarg;
        break;
        case 'f':
            surl = optarg;
        break;
        case 'p':
            protocol = optarg;
        break;
        case 's':
            sscanf(optarg,"%lld",&size);
        break;
        case 'n':
            translate_surl = false;
        break;
        case 'h':
        default:
            print_usage();
            return -1;
        break;
    }
    if((0 == endpoint) || (0 == surl) || (0 == protocol)){
        print_usage();
        return -1;
    }
    
    fprintf(stdout,"contacting %s...\n", endpoint);
    int result = create_file(endpoint,surl,size,protocol,translate_surl);
    fprintf(stdout,"done\n", endpoint);
    
    return result;
}

/*
 * create_file
 * 
 * Create a new File
 */
int create_file(const char * endpoint, const char * surl, LONG64 size, const char * protocol, bool translate_surl){
      
    // Create Fas Proxy Object
    srm::ISRM srmService;
    
    // Set End point
    srmService.endpoint = endpoint;

    if(0 == strncasecmp(endpoint,"httpg://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK;
        soap_register_plugin_arg (srmService.soap, client_cgsi_plugin, &flags);
    } else if(0 == strncasecmp(endpoint,"https://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_SSL_COMPATIBLE;
        soap_register_plugin_arg (srmService.soap, client_cgsi_plugin, &flags);
    }

    std::string p_surl_str;
    if(false == translate_surl) {
        p_surl_str = surl;
    } else {
        std::string surl_str = surl;
    
        // Find first occurrence of ://
        int off = surl_str.find("://");
        if(std::string::npos == off){
            fprintf(stderr,"Invalid SURL %s\n",surl_str.c_str());
            return -1;         
        }
        off +=3;
        
        // Check the Surl To avoid problem with the SRM
        p_surl_str.reserve(surl_str.length() + 1);
        p_surl_str.append(surl_str,0,off);
        std::string::iterator s_it = surl_str.begin() + off;
        std::string::iterator s_end = surl_str.end();
        std::string::iterator p_it = p_surl_str.begin() + off - 1;
        for(; s_it != s_end; ++s_it){
           if((*s_it != '/') || (*p_it != '/')){
                p_surl_str.append(s_it, s_it + 1);
                ++p_it;
            }
        }
        ++p_it;
        p_surl_str.erase(p_it,p_surl_str.end());
        
        fprintf(stdout,"SURL %s translated into %s\n",surl_str.c_str(),p_surl_str.c_str());
    }
    char * processed_surl = new char [p_surl_str.length() + 1];
    strcpy(processed_surl,p_surl_str.c_str());
    
    srm::ArrayOfstring surlarray;
    srm::ArrayOfstring protoarray;
    srm::srm2__RequestStatus *reqstatp;
    srm::ArrayOfstring   srcarray;
    srm::ArrayOflong     sizearray;
    srm::ArrayOfboolean  permarray;
        
    // Set the SURL array
    surlarray.__ptr = (char **)&(processed_surl);
    surlarray.__size = 1;

    // Set the Protocol array
    protoarray.__ptr = (char**)(&protocol);
    protoarray.__size = 1;
    
    // Set the Source array
    srcarray.__ptr = (char **)&processed_surl;
    srcarray.__size = 1;
       
    // Set the Size array
    sizearray.__ptr = &size;
    sizearray.__size = 1;
        
    // Set the Permanent Flag array
    bool setperm = true;
    permarray.__ptr = &setperm;
    permarray.__size = 1;
        
    struct srm::srm__putResponse outp;
        
    // Invoke SRM Put
    if(SOAP_OK != srmService.srm__put(&surlarray, &srcarray, &sizearray, &permarray, &protoarray, GSOAP_RETVAL(outp))) {
        fprintf(stderr,"Cannot Contact SRM Service: Error in srm__put - %s\n",srmService.soap->fault->faultcode);
        soap_print_fault (srmService.soap, stderr);
        delete [] processed_surl;
        return -1;
    }
    fprintf(stdout,"Request to put SURL %s succeeded\n",processed_surl);
    delete [] processed_surl;
    processed_surl = 0;
        
    reqstatp = outp._Result;

    fprintf(stdout,"SRM Put Succeeded\n");
    
    if (reqstatp->fileStatuses == NULL) {
        fprintf(stderr,"No Protocol Support for given file");
        return -1;
    }
    
    int reqid = reqstatp->requestId;
    // Wait the file is "ready"
    // -------------------------------------------------------------------------
    int r = 0;
    while ((strcasecmp (reqstatp->state, "pending") == 0 ) &&
           (strcasecmp (GSOAP_FIELDVAL(reqstatp->fileStatuses->__ptr[0],state), "pending") == 0)) {
            
        fprintf(stdout,"Request status is %s\n",reqstatp->state);
        fprintf(stdout,"File status is %s\n",GSOAP_FIELDVAL(reqstatp->fileStatuses->__ptr[0],state));
    
        // Sleep for a while  
		if(r++ < 10){
			usleep(10000 * r);
		} else {			
			sleep ((reqstatp->retryDeltaTime > 0) ? reqstatp->retryDeltaTime : (r < 20)?(r-10):10);
        }
		
        struct srm::srm__getRequestStatusResponse outq;
        
        // Invoke SRM RequestStatus        
        if(SOAP_OK != srmService.srm__getRequestStatus(reqid, GSOAP_RETVAL(outq))){
            fprintf(stderr,"Failed To Get Status for file %s",surl);
            return -1;
        }
        reqstatp = outq._Result;
    }
    
    // Check Error Codes
    // -------------------------------------------------------------------------
    if (strcasecmp (GSOAP_FIELDVAL(reqstatp->fileStatuses->__ptr[0],state), "ready")) {
        if ((0 != strstr (reqstatp->errorMessage, "does not exist")) ||
            (0 != strstr (reqstatp->errorMessage, "GetStorageInfoFailed")) ||
            (0 != strstr (reqstatp->errorMessage, "nvalid arg"))){
            fprintf(stderr,"Failed To Get The File. Error is %s", reqstatp->errorMessage);
            return -1;
        } else {
            fprintf(stderr,"Failed To Get The File. State is %s - Error is %s", GSOAP_FIELDVAL(reqstatp->fileStatuses->__ptr[0],state), reqstatp->errorMessage);
            return -1;
        }
    }
    
    // Get The File TRUL
    // -------------------------------------------------------------------------
    const char * turl = GSOAP_FIELDVAL(reqstatp->fileStatuses->__ptr[0],TURL);

    fprintf(stdout,"Got the Turl %s\n",turl);
    
    return 0;
}
