/*
 * Copyright (c) Members of the EGEE Collaboration. 2004.
 * See http://www.eu-egee.org/partners/ for details on the copyright holders.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 *      GLite Data SRM API C module: SrmTest class
 *
 *      Authors: Paolo Badino <paolo.badino@cern.ch>
 *      Version info: $Id: SrmTest.cxx,v 1.11 2007/04/04 15:43:56 knienart Exp $ 
 *      Release: $Name:  $
 *
 */
#include <stdlib.h>
#include <stdio.h>
#if _GSOAP_VERSION >= 0x020700
#include "glite/data/srm/cpp/envH.h"
#endif
#include "glite/data/srm/cpp/srmH.h"
#include "glite/data/srm/cpp/srm.nsmap"

/*
 * Include CGSI GSOAP plugin
 */
extern "C"{
#include "cgsi_plugin.h"
}

/**
 * GSOAP Name Wrappers. This code is needed if you want to compile with 
 * GSOAP 2.3 and 2.6
 */
#if _GSOAP_WSDL2H_VERSION < 0x020600
/*
 * Wraps GSOAP 2.3 File names
 */
#include "glite/data/srm/cpp/srmsoapSRMServerV1Proxy.h"
namespace srm{
typedef soapSRMServerV1                 ISRM;
typedef tns__pingResponse               srm__pingResponse;
}
#define GSOAP_RETVAL(a) &a
#else
/*
 * Wraps GSOAP 2.6 File names
 */
#include "glite/data/srm/cpp/srmISRMProxy.h"
#define GSOAP_RETVAL(a) a
#endif

#if _GSOAP_VERSION < 0x020600
/*
 *  GSOAP 2.3 specifics
 */
#define srm__ping                       ping

#else
/*
 *  GSOAP 2.6 specifics
 */
#endif

namespace {
const char * PROGNAME   = "glite_data_srm_api_cpp_test";
const char * PROGVERS   = "0.1";
const char * AUTHOR     = "(C) Paolo Badino - paolo.badino@cern.ch";
}

/*
 * Forward Declaration
 */
int ping(const char * endpoint);
   
/*
 * print_usage
 * 
 * Print Usage
 */
void print_usage(){
    fprintf(stderr,"\n");
    fprintf(stderr,"<%s> Version %s by %s\n",PROGNAME,PROGVERS,AUTHOR);
    fprintf(stderr,"\nUsage:\n");
    fprintf(stderr,"%s -e endpoint\n",PROGNAME);
    fprintf(stderr,"where:\n");
    fprintf(stderr,"\t-e endpoint  : the SRM service endpoint\n");
    fprintf(stderr,"\n");
}

/*
 * main
 * 
 * Test main Function
 */
int main(int argc, char * argv[]){
 
    char * endpoint  = 0;
    
    int c;
    while ((c = getopt (argc, argv, "he:")) != -1)
    switch (c){
        case 'e':
            endpoint = optarg;
        break;
        case 'h':
        default:
            print_usage();
            return -1;
        break;
    }
    if(0 == endpoint){
        print_usage();
        return -1;
    }
    
    fprintf(stdout,"contacting %s...\n", endpoint);
    int result = ping(endpoint);
    fprintf(stdout,"done\n", endpoint);
    
    return result;
}

/*
 * png
 * 
 * Invoke Ping
 */
int ping(const char * endpoint){
      
    // Create Fas Proxy Object
    srm::ISRM srmService;
    
    // Set End point
    srmService.endpoint = endpoint;
    
    if(0 == strncasecmp(endpoint,"httpg://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK;
        soap_register_plugin_arg (srmService.soap, client_cgsi_plugin, &flags);
    } else if(0 == strncasecmp(endpoint,"https://",8)){
        int flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_SSL_COMPATIBLE;
        soap_register_plugin_arg (srmService.soap, client_cgsi_plugin, &flags);
    }
    
#if _GSOAP_VERSION >= 0x020700
    bool ping_out;
#else
    srm::srm__pingResponse ping_out;
#endif

    // Invoke Service ping
    // -------------------------------------------------------------------------
    if(SOAP_OK != srmService.srm__ping(GSOAP_RETVAL(ping_out))){
        fprintf(stderr,"Cannot Contact SRM Service: Error in srm__ping - %s\n",srmService.soap->fault->faultcode);
        soap_print_fault (srmService.soap, stderr);
        return -1;
    }
  
    fprintf(stdout,"SRM Ping Succeeded\n");
    
    return 0;
}

